/*
 Fishino Advanced Chat Server

 A more advanced server that distributes any incoming messages
 to all connected clients but the client the message comes from.
 To use telnet to  your device's IP address and type.
 You can see the client's input in the serial monitor as well.
 Using an Arduino Wiznet Ethernet shield.

 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 * Analog inputs attached to pins A0 through A5 (optional)

 created 18 Dec 2009
 by David A. Mellis
 modified 9 Apr 2012
 by Tom Igoe
 redesigned to make use of operator== 25 Nov 2013
 by Norbert Truchsess
 adapted to Fishino 16 Ago 2015
 by Massimo Del Fedele

 */

#include <Flash.h>
#include <FishinoUdp.h>
#include <FishinoSockBuf.h>
#include <SPI.h>
#include <Fishino.h>

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CONFIGURATION DATA		-- ADAPT TO YOUR NETWORK !!!
// DATI DI CONFIGURAZIONE	-- ADATTARE ALLA PROPRIA RETE WiFi !!!

// here pur SSID of your network
// inserire qui lo SSID della rete WiFi
#define MY_SSID	""

// here put PASSWORD of your network. Use "" if none
// inserire qui la PASSWORD della rete WiFi -- Usare "" se la rete non ￨ protetta
#define MY_PASS	""

// here put required IP address of your Fishino
// comment out this line if you want AUTO IP (dhcp)
// NOTE : if you use auto IP you must find it somehow !
// inserire qui l'IP desiderato per il fishino
// commentare la linea sotto se si vuole l'IP automatico
// nota : se si utilizza l'IP automatico, occorre un metodo per trovarlo !
#define IPADDR	192, 168, 1, 251


//                    END OF CONFIGURATION DATA                      //
//                       FINE CONFIGURAZIONE                         //
///////////////////////////////////////////////////////////////////////

// define ip address if required
#ifdef IPADDR
IPAddress ip(IPADDR);
#endif


// telnet defaults to port 23
// porta telnet standard 23
FishinoServer server(23);

// 4 clients max
// massimo 4 clients
FishinoClient clients[4];

void printWifiStatus()
{
	// print the SSID of the network you're attached to:
	// stampa lo SSID della rete:
	Serial.print("SSID: ");
	Serial.println(Fishino.SSID());

	// print your WiFi shield's IP address:
	// stampa l'indirizzo IP della rete:
	IPAddress ip = Fishino.localIP();
	Serial.print("IP Address: ");
	Serial.println(ip);

	// print the received signal strength:
	// stampa la potenza del segnale di rete:
	long rssi = Fishino.RSSI();
	Serial.print("signal strength (RSSI):");
	Serial.print(rssi);
	Serial.println(" dBm");
}

void setup()
{
	// Initialize serial and wait for port to open
	// Inizializza la porta seriale e ne attende l'apertura
	Serial.begin(115200);
	
	// only for Leonardo needed
	// necessario solo per la Leonardo
	while (!Serial)
		;

	// initialize SPI
	// inizializza il modulo SPI
	SPI.begin();
	SPI.setClockDivider(SPI_CLOCK_DIV2);
	
	// reset and test WiFi module
	// resetta e testa il modulo WiFi
	while(!Fishino.reset())
		Serial << F("Fishino RESET FAILED, RETRYING....\n");
	Serial << F("Fishino WiFi RESET OK\n");

	// go into station mode
	// imposta la modalità stazione
	Fishino.setMode(STATION_MODE);

	Serial.println("Fishino advanced chat server");

	// try forever to connect to AP
	// tenta la connessione finchè non riesce
	while(!Fishino.begin(MY_SSID, MY_PASS) == STATION_GOT_IP)
		Serial << F("ERROR CONNECTING TO AP, RETRYING....\n");
	Serial << F("SUCCESFULLY CONNECTED TO AP\n");
	
	// setup IP or start DHCP client
	// imposta l'IP statico oppure avvia il client DHCP
#ifdef IPADDR
	Fishino.config(ip);
#else
	Fishino.staStartDHCP();
#endif

	// print connection status on serial port
	// stampa lo stato della connessione sulla porta seriale
	printWifiStatus();

	// start listening for clients
	// inizia l'attesa delle connessioni
	server.begin();

	// show current IP on serial port
	// visualizza l'IP corrente sulla porta seriale
	Serial.print("Chat server address:");
	Serial.println(Fishino.localIP());
}

void loop()
{
	// wait for a new client:
	// attende nuovi clienti
	FishinoClient client = server.available();

	// when the client sends the first byte, say hello:
	// quando un cliente invia il primo byte, saluta
	if (client)
	{

		boolean newClient = true;
		for (byte i=0;i<4;i++)
		{
			//check whether this client refers to the same socket as one of the existing instances:
			if (clients[i]==client)
			{
				newClient = false;
				break;
			}
		}

		if (newClient)
		{
			//check which of the existing clients can be overridden:
			for (byte i=0;i<4;i++)
			{
				if (!clients[i] && clients[i]!=client)
				{
					clients[i] = client;
					// clead out the input buffer:
					client.flush();
					Serial.println("We have a new client");
					client.print("Hello, client number: ");
					client.print(i);
					client.println();
					break;
				}
			}
		}

		if (client.available() > 0)
		{
			// read the bytes incoming from the client:
			char thisChar = client.read();
			// echo the bytes back to all other connected clients:
			for (byte i=0;i<4;i++)
			{
				if (clients[i] && (clients[i]!=client))
				{
					clients[i].write(thisChar);
				}
			}
			// echo the bytes to the server as well:
			Serial.write(thisChar);
		}
	}
	for (byte i=0;i<4;i++)
	{
		if (!(clients[i].connected()))
		{
			// client.stop() invalidates the internal socket-descriptor, so next use of == will allways return false;
			clients[i].stop();
		}
	}
}
