/*
  WiFi Web Server using Fishino as an Access Point

 This example creates a WiFi infrastructure using Fishino's soft Ap capabilities
 The AP as an SSID named FISHINO and no password (open network)
 
 Circuit:
 * Analog inputs attached to pins A0 through A5 (optional)

 created 13 July 2010
 by dlf (Metodo2 srl)
 modified 31 May 2012
 by Tom Igoe
 adapted to Fishino 16 Ago 2015
 by Massimo Del Fedele

 */

#include <Flash.h>
#include <FishinoUdp.h>
#include <FishinoSockBuf.h>
#include <Fishino.h>
#include <SPI.h>

///////////////////////////////////////////////////////////////////////
//           CONFIGURATION DATA -- ADAPT TO YOUR NETWORK             //
//     CONFIGURAZIONE SKETCH -- ADATTARE ALLA PROPRIA RETE WiFi      //

// WiFi SSID and PASSWORD
// you can change both, they'll be used to setup the access point
// WiFi SSID e PASSWORD
// potete cambiarle entrambe, verranno utilizzate per la creazione dell'infrastruttura WiFi
#define My_SSID	"FISHINO"
#define My_PASS	""

//                    END OF CONFIGURATION DATA                      //
//                       FINE CONFIGURAZIONE                         //
///////////////////////////////////////////////////////////////////////

FishinoServer server(80);

void setup()
{
	//Initialize serial and wait for port to open:
	// apre la porta seriale e ne attende l'apertura
	Serial.begin(115200);

	// wait for serial port to connect. Needed for Leonardo only
	// attende l'apertura della porta seriale. Necessario solo per le boards Leonardo
	while (!Serial)
		;

	// initialize SPI
	// inizializza il modulo SPI
	SPI.begin();
	SPI.setClockDivider(SPI_CLOCK_DIV2);
	
	// reset and test wifi module
	// resetta e testa il modulo WiFi
	if(Fishino.reset())
		Serial << F("Fishino WiFi RESET OK\r\n");
	else
	{
		Serial << F("Fishino RESET FAILED\r\n");
		
		// stop forever
		// attende per sempre
		while(true)
			;
	}

	Serial << F("Fishino WiFi AP web server\r\n");
	
	// setup station+AP mode
	// imposta la modalitè STAZIONE+AP
	Serial << F("Setting mode SOFTAP_MODE\r\n");
	Fishino.setMode(SOFTAP_MODE);

	// stop AP DHCP server
	Serial << F("Stopping DHCP server\r\n");
	Fishino.softApStopDHCPServer();
	
	// setup access point parameters
	// imposta i parametri dell'access point
	Serial << F("Setting AP IP info\r\n");
	Fishino.setApIPInfo(
		IPAddress(192, 168, 100, 1),
		IPAddress(192, 168, 100, 1),
		IPAddress(255, 255, 255, 0)
	);

	Serial << F("Setting AP WiFi parameters\r\n");
	Fishino.softApConfig(My_SSID, My_PASS, 1, false);
	
	// restart DHCP server
	Serial << F("Starting DHCP server\r\n");
	Fishino.softApStartDHCPServer();
	
	// start listening for clients
	// inizia l'attesa delle connessioni
	server.begin();
}


void loop()
{
	// wait for a new client:
	// attende nuovi clienti
	FishinoClient client = server.available();
	
	if (client)
	{
		Serial << F("new client\r\n");
		
		// an http request ends with a blank line
		// ogni richiesta http termina con una linea vuota
		boolean currentLineIsBlank = true;
		while (client.connected())
		{
			if (client.available())
			{
				char c = client.read();
				Serial.write(c);
				
				// if you've gotten to the end of the line (received a newline
				// character) and the line is blank, the http request has ended,
				// so you can send a reply
				// se si è arrivati a fine linea (carattere 'newline' ricevuto
				// e la linea è vuota, la richiesa http è terminata
				// quindi è possibile inviera una risposta
				if (c == '\n' && currentLineIsBlank)
				{
					// send a standard http response header
					// invia uno header standard http
					client.println("HTTP/1.1 200 OK");
					client.println("Content-Type: text/html");
					client.println("Connection: close");  // the connection will be closed after completion of the response
					client.println("Refresh: 5");  // refresh the page automatically every 5 sec
					client.println();
					client.println("<!DOCTYPE HTML>");
					client.println("<html>");
					
					// output the value of each analog input pin
					// invia il valore di tutti i pins analogici
					for (int analogChannel = 0; analogChannel < 6; analogChannel++)
					{
						int sensorReading = analogRead(analogChannel);
						client.print("analog input ");
						client.print(analogChannel);
						client.print(" is ");
						client.print(sensorReading);
						client.println("<br />");
					}
					client.println("</html>");
					break;
				}
				if (c == '\n')
				{
					// you're starting a new line
					// inizio di una nuova linea
					currentLineIsBlank = true;
				}
				else if (c != '\r')
				{
					// you've gotten a character on the current line
					// sono stati ricevuti dei caratteri nella linea corrente
					currentLineIsBlank = false;
				}
			}
		}
		// give the web browser time to receive the data
		// lascia tempo al browser per ricevere i dati
		delay(1);

		// close the connection:
		// chiudi la connessione
		client.stop();
		Serial.println("client disonnected");
	}
}
